@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 2 -*- */
 /* logout.c - Ask user useful logout questions.
 
    Written by Owen Taylor <otaylor@redhat.com>
@@ -21,12 +22,15 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkinvisible.h>
 #include <gdk/gdkx.h>
 
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <gconf/gconf-client.h>
+
+#include <libgnomeui/gnome-help.h>
 
 #include "ice.h"
 #include "logout.h"
@@ -34,7 +38,21 @@
 #include "util.h"
 #include "gdm-logout-action.h"
 #include "gsm-multiscreen.h"
-#include "egg-screen-help.h"
+
+enum
+{
+  OPTION_LOGOUT,
+  OPTION_HALT,
+  OPTION_REBOOT
+};
+
+static GConfEnumStringPair logout_options_lookup_table[] =
+{
+  { OPTION_LOGOUT, "logout"   },
+  { OPTION_HALT,   "shutdown" },
+  { OPTION_REBOOT, "restart"  },
+  { 0, NULL }
+};
 
 typedef struct {
   GdkScreen    *screen;
@@ -43,8 +61,8 @@ typedef struct {
   int           rowstride;
   GdkWindow    *root_window;
   GdkWindow    *draw_window;
-  GdkPixbuf    *start_pb, *end_pb, *frame;
-  guchar       *start_p, *end_p, *frame_p;
+  GdkPixbuf    *start_pb, *frame;
+  guchar       *start_p, *frame_p;
   GTimeVal      start_time;
   GdkGC        *gc;
 } FadeoutData;
@@ -53,58 +71,40 @@ static GList *fadeout_windows = NULL;
 
 /* Go for five seconds */
 #define FADE_DURATION 1500.0
+#define SATURATION_TARGET 0.5
 
 static void
 get_current_frame (FadeoutData *fadeout,
 		   double    sat)
 {
-  guchar *sp, *ep, *fp;
+  guchar *sp, *fp;
   int i, j, width, offset;
+  unsigned char lut[256];
+  unsigned int value;
 
   width = fadeout->area.width * 3;
   offset = 0;
+
+  /* compute lookup table */
+  for (value = 0; value < sizeof (lut); value++)
+    {
+      lut[value] = SATURATION_TARGET * value * (1 + sat);
+    }
   
   for (i = 0; i < fadeout->area.height; i++)
     {
       sp = fadeout->start_p + offset;
-      ep = fadeout->end_p   + offset;
       fp = fadeout->frame_p + offset;
 
-      for (j = 0; j < width; j += 3)
+      for (j = 0; j < width; j++)
 	{
-	  guchar r = abs (*(sp++) - ep[0]);
-	  guchar g = abs (*(sp++) - ep[1]);
-	  guchar b = abs (*(sp++) - ep[2]);
-
-	  *(fp++) = *(ep++) + r * sat;
-	  *(fp++) = *(ep++) + g * sat;
-	  *(fp++) = *(ep++) + b * sat;
+	  *(fp++) = lut[*(sp++)];
 	}
 
       offset += fadeout->rowstride;
     }
 }
 
-static void
-darken_pixbuf (GdkPixbuf *pb)
-{
-  int width, height, rowstride;
-  int i, j;
-  guchar *p, *pixels;
-  
-  width     = gdk_pixbuf_get_width (pb) * 3;
-  height    = gdk_pixbuf_get_height (pb);
-  rowstride = gdk_pixbuf_get_rowstride (pb);
-  pixels    = gdk_pixbuf_get_pixels (pb);
-  
-  for (i = 0; i < height; i++)
-    {
-      p = pixels + (i * rowstride);
-      for (j = 0; j < width; j++)
-	p [j] >>= 1;
-    }
-}
-
 static gboolean
 fadeout_callback (FadeoutData *fadeout)
 {
@@ -123,19 +123,8 @@ fadeout_callback (FadeoutData *fadeout)
 
   if (elapsed > FADE_DURATION)
     {
-      gdk_draw_pixbuf (fadeout->draw_window,
-		       fadeout->gc,
-		       fadeout->end_pb,
-		       0, 0,
-		       0, 0,
-		       fadeout->area.width,
-		       fadeout->area.height,
-		       GDK_RGB_DITHER_NONE,
-		       0, 0);
-
       g_object_unref (fadeout->gc);
       g_object_unref (fadeout->start_pb);
-      g_object_unref (fadeout->end_pb);
       g_object_unref (fadeout->frame);
 
       g_free (fadeout);
@@ -204,14 +193,10 @@ fadeout_screen (GdkScreen *screen,
 						    fadeout->area.width,
 						    fadeout->area.height);
   
-  fadeout->end_pb = gdk_pixbuf_copy (fadeout->start_pb);
-  darken_pixbuf (fadeout->end_pb);
-  
   fadeout->frame = gdk_pixbuf_copy (fadeout->start_pb);
   fadeout->rowstride = gdk_pixbuf_get_rowstride (fadeout->start_pb);
 
   fadeout->start_p = gdk_pixbuf_get_pixels (fadeout->start_pb);
-  fadeout->end_p   = gdk_pixbuf_get_pixels (fadeout->end_pb);
   fadeout->frame_p = gdk_pixbuf_get_pixels (fadeout->frame);
   
   values.subwindow_mode = GDK_INCLUDE_INFERIORS;
@@ -265,6 +250,37 @@ make_title_label (const char *text)
   return label;
 }
 
+static int
+get_default_option (void)
+{
+  GConfClient *gconf_client;
+  char        *str;
+  int          option;
+
+  gconf_client = gsm_get_conf_client ();
+  str = gconf_client_get_string (gconf_client, LOGOUT_OPTION_KEY, NULL);
+
+  if (str == NULL || !gconf_string_to_enum (logout_options_lookup_table, str, &option))
+    option = OPTION_LOGOUT;
+
+  g_free (str);
+  return option;
+}
+
+static void
+set_default_option (int option)
+{
+  GConfClient *gconf_client;
+  const char  *str;
+
+  gconf_client = gsm_get_conf_client ();
+
+  str = gconf_enum_to_string (logout_options_lookup_table, option);
+  g_assert (str != NULL);
+
+  gconf_client_set_string (gconf_client, LOGOUT_OPTION_KEY, str, NULL);
+}
+
 static gboolean
 display_gui (void)
 {
@@ -289,6 +305,7 @@ display_gui (void)
   GError *error = NULL;
   GdkScreen *screen;
   int monitor;
+  int selected_option;
 
   gsm_verbose ("display_gui: showing logout dialog\n");
 
@@ -314,7 +331,7 @@ display_gui (void)
     {
       while (1)
 	{
-	  if (gdk_pointer_grab (invisible->window, FALSE, 0,
+	  if (gdk_pointer_grab (invisible->window, TRUE, 0,
 				NULL, NULL, GDK_CURRENT_TIME) == Success)
 	    {
 	      if (gdk_keyboard_grab (invisible->window, FALSE, GDK_CURRENT_TIME)
@@ -363,13 +380,13 @@ display_gui (void)
 
   gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_OK);
   gtk_window_set_screen (GTK_WINDOW (box), screen);
-  gtk_window_set_policy (GTK_WINDOW (box), FALSE, FALSE, TRUE);
+  gtk_window_set_resizable (GTK_WINDOW (box), FALSE);
 
   gtk_container_set_border_width (GTK_CONTAINER (box), 5);
 
   if (!autosave)
     {
-      toggle_button = gtk_check_button_new_with_mnemonic (_("_Save current setup"));
+      toggle_button = gtk_check_button_new_with_mnemonic (_("Remember running applications"));
       gtk_widget_show (toggle_button);
       gtk_box_pack_start (GTK_BOX (vbox),
 			  toggle_button,
@@ -385,6 +402,8 @@ display_gui (void)
       GtkWidget *action_vbox, *hbox;
       GtkWidget *category_vbox;
       GtkWidget *r;
+
+      selected_option = get_default_option ();
       
       category_vbox = gtk_vbox_new (FALSE, 6);
       gtk_box_pack_start (GTK_BOX (vbox), category_vbox, TRUE, TRUE, 0);
@@ -408,12 +427,14 @@ display_gui (void)
       gtk_widget_show (action_vbox);
       
       r = gtk_radio_button_new_with_mnemonic (NULL, _("_Log out"));
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r), (selected_option == OPTION_LOGOUT));
       gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
       gtk_widget_show (r);
 
       if (halt_supported)
 	{
 	  r = halt = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (r), _("Sh_ut down"));
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r), (selected_option == OPTION_HALT));
 	  gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
 	  gtk_widget_show (r);
 	}
@@ -421,6 +442,7 @@ display_gui (void)
       if (reboot_supported)
 	{
 	  r = reboot = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (r), _("_Restart the computer"));
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r), (selected_option == OPTION_REBOOT));
 	  gtk_box_pack_start (GTK_BOX (action_vbox), r, FALSE, FALSE, 0);
 	  gtk_widget_show (r);
 	}
@@ -462,6 +484,13 @@ display_gui (void)
   if (toggle_button)
     save_active = GTK_TOGGLE_BUTTON (toggle_button)->active;
 
+  if (reboot_active)
+    selected_option = OPTION_REBOOT;
+  else if (halt_active)
+    selected_option = OPTION_HALT;
+  else
+    selected_option = OPTION_LOGOUT;
+
   gtk_widget_destroy (box);
   gtk_widget_destroy (invisible);
 
@@ -486,6 +515,7 @@ display_gui (void)
 	logout_action = GDM_LOGOUT_ACTION_SHUTDOWN;
       else if (reboot_active)
 	logout_action = GDM_LOGOUT_ACTION_REBOOT;
+      set_default_option (selected_option);
       retval = TRUE;
       break;
     default:
@@ -493,11 +523,11 @@ display_gui (void)
       retval = FALSE;
       break;
     case GTK_RESPONSE_HELP:
-      egg_help_display_desktop_on_screen (NULL, "user-guide",
-					  "user-guide.xml",
-					  "gosgetstarted-73",
-					  screen,
-					  &error);
+      gnome_help_display_desktop_on_screen (NULL, "user-guide",
+					    "user-guide.xml",
+					    "gosgetstarted-73",
+					    screen,
+					    &error);
 
       if (error) 
         {
