@@ -29,13 +29,17 @@
 #include <sys/socket.h>  /* For have_ipv6() */
 #include <netdb.h>
 #include <time.h>
+#include <unistd.h>
+
+#include <glib/gi18n.h>
+
+#include <orbit/orbit.h>
 
 #include <gconf/gconf-client.h>
 
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
-
-#include <libgnomeui/gnome-window-icon.h>
+#include <libgnomeui/gnome-client.h>
+#include <libgnomeui/gnome-ui-init.h>
+#include <libgnome/gnome-config.h>
 
 #include "manager.h"
 #include "ice.h"
@@ -141,9 +145,258 @@ set_gtk1_theme_rcfile (void)
 {
   char *env_string;
 
-  env_string = g_strdup_printf ("GTK_RC_FILES=" SYSCONFDIR "/gtk/gtkrc:%s/.gtkrc-1.2-gnome2", g_get_home_dir ());
+  env_string = g_strdup_printf ("%s/gtk/gtkrc:%s/.gtkrc-1.2-gnome2",
+                                SYSCONFDIR, g_get_home_dir ());
+  g_setenv ("GTK_RC_FILES", env_string, TRUE);
+  g_free (env_string);
+}
 
-  putenv (env_string);
+#define ROOTSESSION_RESPONSE_CONTINUE 1
+#define ROOTSESSION_RESPONSE_QUIT 3
+static gboolean
+gsm_check_for_root (void)
+{
+  GtkWidget *dlg;
+  gint       response;
+
+  if (geteuid () != 0)
+    return FALSE;
+
+  dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
+                                GTK_BUTTONS_NONE,
+                                _("This session is running as a privileged user"));
+  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
+                                            _("Running a session as a privileged user should be avoided for security reasons. If possible, you should log in as a normal user."));
+
+  /* FIXME: would be nice to have a icon for Continue */
+  gtk_dialog_add_buttons (GTK_DIALOG (dlg),
+                          _("_Continue"), ROOTSESSION_RESPONSE_CONTINUE,
+                          GTK_STOCK_QUIT, ROOTSESSION_RESPONSE_QUIT,
+                          NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (dlg),
+                                   ROOTSESSION_RESPONSE_QUIT);
+  gtk_window_set_title (GTK_WINDOW (dlg), "");
+  gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
+
+  response = gtk_dialog_run (GTK_DIALOG (dlg));
+  gtk_widget_destroy (dlg);
+
+  return !(response == ROOTSESSION_RESPONSE_CONTINUE);
+}
+
+#define FALLBACK_TIME_UTILITY "time-admin"
+#define CLOCKSESSION_RESPONSE_IGNORE 1
+#define CLOCKSESSION_RESPONSE_ADJUST 3
+
+struct check_clock {
+  GtkWidget  *dialog;
+  char      **argv_tool;
+  guint       timeout_id;
+  guint       child_id;
+};
+
+static gboolean
+gsm_check_time (void)
+{
+  time_t now;
+
+  now = time (NULL);
+  /* accept a one week error: no need to be too strict */
+  return (now > GNOME_COMPILE_TIME - 3600*24*7);
+}
+
+static void
+gsm_check_clock_set_label (GtkWidget *dialog)
+{
+  char       date[64];
+  char      *text;
+  time_t     now;
+  struct tm *tim;
+
+  now = time (NULL);
+  tim = localtime (&now);
+  strftime (date, sizeof (date), "%x", tim);
+
+  text = g_markup_printf_escaped (_("The session might encounter issues if the computer clock is not properly configured. Please consider adjusting it.\n\nCurrent date is <b>%s</b>."), date);
+  gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
+                                              text);
+  g_free (text);
+}
+
+static gboolean
+gsm_check_clock_config_tool_exists (struct check_clock *check,
+                                    const char         *config_tool)
+{
+  char **argv;
+  char  *path;
+
+  if (!config_tool || config_tool[0] == '\0')
+    return FALSE;
+
+  argv = NULL;
+  if (!g_shell_parse_argv (config_tool, NULL, &argv, NULL))
+    return FALSE;
+
+  if (!(path = g_find_program_in_path (argv [0]))) {
+    g_strfreev (argv);
+    return FALSE;
+  }
+
+  g_free (path);
+
+  check->argv_tool = argv;
+
+  return TRUE;
+}
+
+static void
+gsm_check_clock_main_quit (struct check_clock *check)
+{
+  if (check->dialog)
+    gtk_widget_destroy (check->dialog);
+  check->dialog = NULL;
+
+  if (check->argv_tool)
+    g_strfreev (check->argv_tool);
+  check->argv_tool = NULL;
+
+  if (check->timeout_id)
+    g_source_remove (check->timeout_id);
+  check->timeout_id = 0;
+
+  if (check->child_id)
+    g_source_remove (check->child_id);
+  check->child_id = 0;
+
+  gtk_main_quit ();
+}
+
+static gboolean
+gsm_check_clock_update (gpointer data)
+{
+  struct check_clock *check;
+
+  check = (struct check_clock *) data;
+
+  if (gsm_check_time ())
+    {
+      gsm_check_clock_main_quit (check);
+      return FALSE;
+    }
+
+  if (check->dialog)
+    gsm_check_clock_set_label (check->dialog);
+
+  return TRUE;
+}
+
+static void
+gsm_check_clock_tool_exited (GPid     pid,
+                             gint     status,
+                             gpointer data)
+{
+  gsm_check_clock_main_quit ((struct check_clock *) data);
+
+  g_spawn_close_pid (pid);
+}
+
+static void
+gsm_check_clock_response (GtkWidget          *dialog,
+                          int                 response,
+                          struct check_clock *check)
+{
+  gtk_widget_destroy (dialog);
+  check->dialog = NULL;
+
+  if (response == CLOCKSESSION_RESPONSE_ADJUST)
+    {
+      GtkWidget *dialog;
+      GPid       pid;
+      GError    *err;
+
+      err = NULL;
+
+      if (g_spawn_async (NULL,
+                         check->argv_tool,
+                         NULL,
+                         G_SPAWN_SEARCH_PATH|G_SPAWN_DO_NOT_REAP_CHILD,
+                         NULL,
+                         NULL,
+                         &pid,
+                         &err))
+        {
+          check->child_id = g_child_watch_add (pid,
+                                               gsm_check_clock_tool_exited,
+                                               check);
+          return;
+        }
+
+	dialog = gtk_message_dialog_new (NULL,
+                                         GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_OK,
+					 _("Failed to launch time configuration tool: %s"),
+					 err->message);
+	g_error_free (err);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
+
+        gtk_dialog_run (GTK_DIALOG (dialog));
+
+        gsm_check_clock_main_quit (check);
+    }
+  else
+    gsm_check_clock_main_quit (check);
+
+}
+
+static void
+gsm_check_clock (void)
+{
+  struct check_clock check;
+
+  if (gsm_check_time ())
+    return;
+
+  check.child_id = 0;
+
+  if (!gsm_check_clock_config_tool_exists (&check, TIME_UTILITY) &&
+      !gsm_check_clock_config_tool_exists (&check, FALLBACK_TIME_UTILITY))
+    check.argv_tool = NULL;
+
+  check.dialog = gtk_message_dialog_new (NULL,
+                                         GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
+                                         GTK_BUTTONS_NONE,
+                                         _("The computer clock appears to be wrong"));
+  gsm_check_clock_set_label (check.dialog);
+
+  if (check.argv_tool)
+    {
+      /* FIXME: would be nice to have a icons for the buttons */
+      gtk_dialog_add_buttons (GTK_DIALOG (check.dialog),
+                              _("_Ignore"), CLOCKSESSION_RESPONSE_IGNORE,
+                              _("_Adjust the Clock"), CLOCKSESSION_RESPONSE_ADJUST,
+                              NULL);
+      gtk_dialog_set_default_response (GTK_DIALOG (check.dialog),
+                                       CLOCKSESSION_RESPONSE_ADJUST);
+    }
+  else
+    {
+      gtk_dialog_add_button (GTK_DIALOG (check.dialog),
+                             GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+    }
+
+  gtk_window_set_title (GTK_WINDOW (check.dialog), "");
+  gtk_window_set_position (GTK_WINDOW (check.dialog), GTK_WIN_POS_CENTER);
+
+  g_signal_connect (check.dialog, "response",
+                    G_CALLBACK (gsm_check_clock_response), &check);
+
+  gtk_widget_show (check.dialog);
+
+  check.timeout_id = g_timeout_add (1000 * 60,
+                                    gsm_check_clock_update, &check);
+
+  gtk_main ();
 }
 
 static void
@@ -313,8 +566,6 @@ gsm_shutdown_gconfd (void)
 int
 main (int argc, char *argv[])
 {
-  char *ep;
-  char *session_name_env;
   Session *the_session;
   gboolean splashing;
   gboolean a_t_support;
@@ -326,7 +577,7 @@ main (int argc, char *argv[])
   GOptionContext *goption_context;
   gboolean dbus_daemon_owner;
   
-  if (getenv ("GSM_VERBOSE_DEBUG"))
+  if (g_getenv ("GSM_VERBOSE_DEBUG"))
     gsm_set_verbose (TRUE);
 
   /* Help eradicate the critical warnings in unstable releases of GNOME */
@@ -337,7 +588,7 @@ main (int argc, char *argv[])
       major = atoi (versions [1]);
       if ((major % 2) != 0)
 	{
-	  putenv ("G_DEBUG=fatal_criticals");
+          g_setenv ("G_DEBUG", "fatal_criticals", FALSE);
 	  g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);
 	}
     }
@@ -357,6 +608,11 @@ main (int argc, char *argv[])
    */
   gtk_init (&argc, &argv);
 
+  gtk_window_set_default_icon_name ("session-properties");
+
+  if (gsm_check_for_root ())
+    return 0;
+
   /* We need to do this as early as possible */
   gsm_set_display_properties ();
   
@@ -392,7 +648,8 @@ main (int argc, char *argv[])
       gsm_at_set_gtk_modules ();
     }
 
-  goption_context = g_option_context_new (_("- Manage the GNOME session"));
+  goption_context = g_option_context_new (N_("- Manage the GNOME session"));
+  g_option_context_set_translation_domain (goption_context, GETTEXT_PACKAGE);
   g_option_context_add_main_entries (goption_context, options, GETTEXT_PACKAGE);
 
   gnome_program_init("gnome-session", VERSION, LIBGNOMEUI_MODULE, argc, argv, 
@@ -409,16 +666,13 @@ main (int argc, char *argv[])
   }
 
   initialize_ice ();
-  fprintf (stderr, "SESSION_MANAGER=%s\n", getenv ("SESSION_MANAGER"));
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-session.png");
-
+  fprintf (stderr, "SESSION_MANAGER=%s\n", g_getenv ("SESSION_MANAGER"));
 
   /* Make sure children see the right value for DISPLAY.  This is
      useful if --display was specified on the command line.  */
   display_str = gdk_get_display ();
-  ep = g_strconcat ("DISPLAY=", display_str, NULL);
+  g_setenv ("DISPLAY", display_str, TRUE);
   g_free (display_str);
-  putenv (ep);
 
   ignore (SIGPIPE);
 
@@ -426,6 +680,9 @@ main (int argc, char *argv[])
   gsm_keyring_daemon_start ();
   dbus_daemon_owner = gsm_dbus_daemon_start ();
   gsm_gsd_start ();
+
+  /* FIXME is it useful to do it so late? gsd migt not work, eg */
+  gsm_check_clock ();
   
   /* Read the rest of config options */  
 
@@ -472,9 +729,8 @@ main (int argc, char *argv[])
 		  g_free (session_name);
 	  session_name = g_strdup (FAILSAFE_SESSION);
   }
-  
-  session_name_env = g_strconcat ("GNOME_DESKTOP_SESSION_ID=", session_name, NULL);
-  putenv (session_name_env);
+
+  g_setenv ("GNOME_DESKTOP_SESSION_ID", session_name, TRUE);
   the_session = read_session (session_name);
 
   gsm_sound_login ();
@@ -486,9 +742,6 @@ main (int argc, char *argv[])
 
   start_session (the_session);
 
-  if (a_t_support) /* the ATs are happier if the session has started */
-    gsm_assistive_technologies_start ();
-
   gtk_main ();
 
   gsm_remote_desktop_cleanup ();
